Package: labelled
Type: Package
Title: Manipulating Labelled Data
Version: 2.13.0
Maintainer: Joseph Larmarange <joseph@larmarange.net>
Authors@R: c(
  person("Joseph", "Larmarange", email = "joseph@larmarange.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-7097-700X")),
  person("Daniel", "Ludecke", role = "ctb"),
  person("Hadley", "Wickham", role = "ctb"),
  person("Michal", "Bojanowski", role = "ctb"),
  person("François", "Briatte", role = "ctb")
  )
Description: Work with labelled data imported from 'SPSS'
    or 'Stata' with 'haven' or 'foreign'. This package
    provides useful functions to deal with "haven_labelled" and
    "haven_labelled_spss" classes introduced by 'haven' package.
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.0)
Imports: haven (>= 2.4.1), dplyr (>= 1.0.0), lifecycle, rlang, vctrs,
        stringr, tidyr, tidyselect
Suggests: testthat, knitr, rmarkdown, questionr, snakecase, utf8, covr,
        spelling
Enhances: memisc
URL: https://larmarange.github.io/labelled/,
        https://github.com/larmarange/labelled
BugReports: https://github.com/larmarange/labelled/issues
VignetteBuilder: knitr
LazyData: true
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
RdMacros: lifecycle
Language: en-US
Author: Joseph Larmarange [aut, cre] (<https://orcid.org/0000-0001-7097-700X>),
  Daniel Ludecke [ctb],
  Hadley Wickham [ctb],
  Michal Bojanowski [ctb],
  François Briatte [ctb]
Built: R 4.4.0; ; 2024-04-28 01:25:33 UTC; unix
