Package: labelled
Type: Package
Title: Manipulating Labelled Data
Version: 2.16.0
Maintainer: Joseph Larmarange <joseph@larmarange.net>
Authors@R: c(
  person("Joseph", "Larmarange", email = "joseph@larmarange.net", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-7097-700X")),
  person("Daniel", "Ludecke", role = "ctb"),
  person("Hadley", "Wickham", role = "ctb"),
  person("Michal", "Bojanowski", role = "ctb"),
  person("François", "Briatte", role = "ctb")
  )
Description: Work with labelled data imported from 'SPSS'
    or 'Stata' with 'haven' or 'foreign'. This package
    provides useful functions to deal with "haven_labelled" and
    "haven_labelled_spss" classes introduced by 'haven' package.
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.2)
Imports: haven (>= 2.4.1), cli, dplyr (>= 1.1.0), lifecycle, rlang (>=
        1.1.0), vctrs, stringr, tidyr (>= 1.3.0), tidyselect
Suggests: gt, knitr, rmarkdown, purrr, questionr, snakecase, spelling,
        survey, testthat (>= 3.2.0)
Enhances: memisc
URL: https://larmarange.github.io/labelled/,
        https://github.com/larmarange/labelled
BugReports: https://github.com/larmarange/labelled/issues
VignetteBuilder: knitr
LazyData: true
RoxygenNote: 7.3.3
Language: en-US
Config/testthat/edition: 3
Config/Needs/check: memisc
NeedsCompilation: no
Packaged: 2025-10-22 16:33:32 UTC; josep
Author: Joseph Larmarange [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7097-700X>),
  Daniel Ludecke [ctb],
  Hadley Wickham [ctb],
  Michal Bojanowski [ctb],
  François Briatte [ctb]
Repository: CRAN
Date/Publication: 2025-10-22 17:00:02 UTC
Built: R 4.6.0; ; 2026-04-30 06:05:14 UTC; unix
